;; Pac GC - Changer le style de Multi-Lignes ...
;; http://www.cadxp.com/modules.php?op=modload&name=XForum&file=viewthread&tid=30135#pid130971
;; Commande : CMLS

(defun c:CMLS (/ *error* echo osnp slst ss styl ent)
  (vl-load-com)
  (defun *error* (msg)
    (or (not msg)
        (= msg "Fonction annulée")
        (princ (strcat "Erreur: " msg))
    )
    (setvar 'cmdecho echo)
    (setvar 'osmode osnp)
    (vla-EndUndoMark (vla-get-ActiveDocument (vlax-get-acad-object)))
    (princ)
  )
  (setq echo (getvar 'cmdecho)
        osnp (getvar 'osmode)
        slst (gc:GetDictEntries (dictsearch (namedobjdict) "ACAD_MLINESTYLE"))
  )
  (if
    (and
      (setq n  -1
            ss (ssget '((0 . "MLINE")))
      )
      (setq styl (gc:ListBox "Styles de multiligne" "Choisissez un style" (mapcar 'car slst) 0))
    )
     (progn
       (vla-StartUndoMark (vla-get-ActiveDocument (vlax-get-acad-object)))
       (setvar 'cmdecho 0)
       (setvar 'osmode 0)
       (while (setq ent (ssname ss (setq n (1+ n))))
         (setq elst (entget ent))
         (vl-cmdf "_.mline" "_style" styl)
         (mapcar '(lambda (x)
                    (vl-cmdf (trans x 0 1))
                  )
                 (massoc 11 elst)
         )
         (vl-cmdf (if (= 2 (logand 2 (cdr (assoc 71 elst))))
                    "_close"
                    ""
                  )
         )
         (if (setq xd (assoc -3 (entget ent '("*"))))
           (entmod (append (entget (entlast)) (list xd)))
         )
         (entdel ent)
       )
     )
  )
  (*error* nil)
)

;; gc:ListBox par GC
;; Boite de dialogue permettant un ou plusieurs choix dans une liste
;;
;; Arguments
;; title : le titre de la boite de dialogue (chaîne)
;; msg ; message (chaîne), "" ou nil pour aucun
;; lbls : une liste de chaînes (sans doublons)
;; flag : 0 = liste déroulante
;;        1 = liste choix unique
;;        2 = liste choix multipes
;;
;; Retour : l'option (flag = 0 ou 1) ou la liste des options (flag = 2)
;;
;; Exemple d'utilisation
;; (listbox "Présentation" "Choisir une présentation" (layoutlist) 1)

(defun gc:ListBox (title msg lbls flag / tmp file dcl_id choice)
  (setq tmp  (vl-filename-mktemp "tmp.dcl")
        file (open tmp "w")
  )
  (write-line
    (strcat "ListBox:dialog{label=\"" title "\";")
    file
  )
  (if (and msg (/= msg ""))
    (write-line (strcat ":text{label=\"" msg "\";}") file)
  )
  (write-line
    (cond
      ((= 0 flag) "spacer;:popup_list{key=\"lst\";")
      ((= 1 flag) "spacer;:list_box{key=\"lst\";")
      (T "spacer;:list_box{key=\"lst\";multiple_select=true;")
    )
    file
  )
  (write-line "}spacer;ok_cancel;}" file)
  (close file)
  (setq dcl_id (load_dialog tmp))
  (if (not (new_dialog "ListBox" dcl_id))
    (exit)
  )
  (start_list "lst")
  (mapcar 'add_list lbls)
  (end_list)
  (action_tile
    "accept"
    "(or (= (get_tile \"lst\") \"\")
    (if (= 2 flag) (progn
    (foreach n (gc:str2lst (get_tile \"lst\") \" \")
    (setq choice (cons (nth (atoi n) lbls) choice)))
    (setq choice (reverse choice)))
    (setq choice (nth (atoi (get_tile \"lst\")) lbls))))
    (done_dialog)"
  )
  (start_dialog)
  (unload_dialog dcl_id)
  (vl-file-delete tmp)
  choice
)

;; gc:str2lst
;; Transforme un chaine avec séparateur en liste de chaines
;;
;; Arguments
;; str : la chaine à transformer en liste
;; sep : le séparateur
;;
;; Exemples
;; (str2lst "a b c" " ") -> ("a" "b" "c")
;; (str2lst "1,2,3" ",") -> ("1" "2" "3")
;; (mapcar 'read (str2lst "1,2,3" ",")) -> (1 2 3)

(defun gc:str2lst (str sep / pos)
  (if (setq pos (vl-string-search sep str))
    (cons (substr str 1 pos)
          (gc:str2lst (substr str (+ (strlen sep) pos 1)) sep)
    )
    (list str)
  )
)

;; gc:GetDictEntries
;; Retourne la liste des entrées du dictionnaire
;; sous forme de paires pointées (Nom . ENAME)
;;
;; Argument : dict le dictionnaire (ENAME ou liste DXF)

(defun gc:GetDictEntries (dict / result)
  (and (= (type dict) 'ENAME) (setq dict (entget dict)))
  (while
    (setq dict (vl-member-if (function (lambda (x) (= (car x) 3))) (cdr dict)))
     (setq result (cons (cons (cdar dict) (cdadr dict)) result))
  )
  (reverse result)
)

;; massoc
;; Retourne la liste de toutes les valeurs pour le code spécifié
;; dans une liste d'association
;;
;; Arguments
;; code : le code de groupe pour les entrées
;; alst : la liste d'association

(defun massoc (code alst / pair)
  (if (setq alst (member (assoc code alst) alst))
    (cons (cdar alst) (massoc code (cdr alst)))
  )
)
 